﻿using System;

namespace Platform.Geo
{
    public class GeoRectangle
    {
        public GeoPosition NorthwestCorner { get; set; }
        public GeoPosition SoutheastCorner { get; set; }

        #region Tools

        public double Width
        {
            get { return SoutheastCorner.Longitude - NorthwestCorner.Longitude; }
        }

        public double Height
        {
            get { return NorthwestCorner.Latitude - SoutheastCorner.Latitude; }
        }

        public void Grow( double latitude, double longitude )
        {
            NorthwestCorner.Longitude -= longitude;
            SoutheastCorner.Longitude += longitude;
            NorthwestCorner.Latitude += longitude;
            SoutheastCorner.Latitude -= longitude;
        }

        /// <summary>
        /// Union of two rectangles
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public GeoRectangle Union( GeoRectangle other )
        {
            return new GeoRectangle
            {
                NorthwestCorner = new GeoPosition
                {
                    Latitude = Math.Max(this.NorthwestCorner.Latitude, other.NorthwestCorner.Latitude),
                    Longitude = Math.Min(this.NorthwestCorner.Longitude, other.NorthwestCorner.Longitude)
                },
                SoutheastCorner = new GeoPosition
                {
                    Latitude = Math.Min(this.SoutheastCorner.Latitude, other.SoutheastCorner.Latitude),
                    Longitude = Math.Max(this.SoutheastCorner.Longitude, other.SoutheastCorner.Longitude)
                }
            };             
        }        

        public GeoRectangle Union( GeoPosition point )
        {
            return new GeoRectangle
            {
                NorthwestCorner = new GeoPosition
                {
                    Latitude = Math.Max(this.NorthwestCorner.Latitude, point.Latitude),
                    Longitude = Math.Min(this.NorthwestCorner.Longitude, point.Longitude)
                },
                SoutheastCorner = new GeoPosition
                {
                    Latitude = Math.Min(this.SoutheastCorner.Latitude, point.Latitude),
                    Longitude = Math.Max(this.SoutheastCorner.Longitude, point.Longitude)
                }
            };
        }

        /// <summary>
        /// Convert to polygon
        /// </summary>
        /// <returns></returns>
        public GeoPolygon ToPolygon()
        {
            return new GeoPolygon
            {
                Points = new GeoPosition[][][]
                {
                    new GeoPosition[][]
                    {
                        new GeoPosition[]
                        {
                            NorthwestCorner,
                            new GeoPosition { Latitude = SoutheastCorner.Latitude, Longitude = NorthwestCorner.Longitude },
                            SoutheastCorner,
                            new GeoPosition { Latitude = NorthwestCorner.Latitude, Longitude = SoutheastCorner.Longitude }
                        }
                    }
                }
            };
        }

        #endregion
    }
}
